use crate::{Context, Error};
use cached::proc_macro::cached;
use serde_json::{
    Map,
    Value::{self, Object},
};
use std::fs;
use rand::seq::IteratorRandom;


async fn autocomplete_command<'a>(
    _ctx: Context<'_>,
    partial: &'a str,
) -> impl Iterator<Item = String> + 'a {
    let res: Vec<String> = get_coms()
        .keys()
        .filter(move |e| e.contains(&partial))
        .map(|e| e.to_string())
        .collect();
    res.into_iter()
}

#[cached(time = 600)]
fn get_coms() -> Map<String, Value> {
    let path = "./data/commands.json";
    let data = fs::read_to_string(path).expect("Unable to read file");
    let res: serde_json::Value = serde_json::from_str(&data).expect("Unable to parse");
    if let Object(things) = res {
        return things;
    } else {
        return Map::<String, Value>::new();
    }
}

// F1TV links throughout the seasons
#[poise::command(slash_command)]
pub async fn coms(
    ctx: Context<'_>,
    #[description = "Which command to execute?"]
    #[autocomplete = "autocomplete_command"]
    command: String,
) -> Result<(), Error> {
    match get_coms().get(&command) {
        None => {
            ctx.say(format!("Command `{}` not found", command)).await?;
        }
        Some(com) => {
            ctx.say(sky(com.as_str().unwrap())).await?;
        }
    }

    Ok(())
}


fn sky(input: &str) -> String {
    let result: String = input.to_string();
    let mut rng = rand::thread_rng();
    loop {
        match sky_open(&result) {
            None => return result,
            Some((left, end)) => {
                match sky_closed(&end) {
                    None => return result,
                    Some((mid, right)) => return sky(&format!("{}{}{}", left, mid.split('|').choose(&mut rng).unwrap(), right))
                };
            }
        }
    }
}

fn sky_open(input: &str) -> Option<(String, String)> {
    match input.rsplit_once('{') {
        None => (return None),
        Some((left, end)) => {
            if left.ends_with('\\') {
                match sky_open(left) {
                    None => (return None),
                    Some((left, right)) => {
                        let mut end: String = end.to_string();
                        end.push('{');
                        end.push_str(right.as_str());
                        return Some((left.to_owned(), end))
                    }
                };
            } else {
                return Some((left.to_owned(), end.to_owned()));
            }
        }
    }
}

fn sky_closed(input: &str) -> Option<(String, String)> {
    match input.split_once('}') {
        None => (return None),
        Some((left, end)) => {
            if left.ends_with('\\') {
                match sky_closed(end) {
                    None => (return None),
                    Some((mid, right)) => {
                        let mut start: String = left.to_string();
                        start.push('}');
                        start.push_str(mid.as_str());
                        return Some((start, right.to_owned()))
                    }
                };
            } else {
                return Some((left.to_owned(), end.to_owned()));
            }
        }
    }
}